@@ -1,5 +1,5 @@
 #!/usr/bin/env python
-# Copyright (C) 2013-2016  Vincent Pelletier <plr.vincent@gmail.com>
+# Copyright (C) 2013-2018  Vincent Pelletier <plr.vincent@gmail.com>
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -14,12 +14,13 @@
 # You should have received a copy of the GNU Lesser General Public
 # License along with this library; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+from __future__ import print_function
 import usb1
 
 def main():
     with usb1.USBContext() as context:
         for device in context.getDeviceIterator(skip_on_error=True):
-            print 'ID %04x:%04x' % (device.getVendorID(), device.getProductID()), '->'.join(str(x) for x in ['Bus %03i' % (device.getBusNumber(), )] + device.getPortNumberList()), 'Device', device.getDeviceAddress()
+            print('ID %04x:%04x' % (device.getVendorID(), device.getProductID()), '->'.join(str(x) for x in ['Bus %03i' % (device.getBusNumber(), )] + device.getPortNumberList()), 'Device', device.getDeviceAddress())
 
 if __name__ == '__main__':
     main()
