@@ -1,4 +1,4 @@
-# Copyright (C) 2010-2016  Vincent Pelletier <plr.vincent@gmail.com>
+# Copyright (C) 2010-2018  Vincent Pelletier <plr.vincent@gmail.com>
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -149,8 +149,69 @@ if sys.version_info[0] == 3:
     xrange = range
     long = int
     # pylint: enable=redefined-builtin
+    integer_memoryview = memoryview
 else:
     BYTE = '\x00'
+    # Work around python2's memoryview, which only accepts & generates strings.
+    # For consistency between async control and other async transfers.
+    # Python 2.7 will not be fixed, so wrap its memoryview.
+    # Breaks the no-copy promise, but control transfer performance should
+    # matter less than other types.
+    class integer_memoryview(object):
+        def __init__(self, view):
+            if not isinstance(view, memoryview):
+                view = memoryview(view)
+            self.__view = view
+
+        # Many boring magic methods, just to mimic memoryview
+        def __eq__(self, other):
+            return self.__view == other
+
+        def __ge__(self, other):
+            return self.__view >= other
+
+        def __gt__(self, other):
+            return self.__view > other
+
+        def __le__(self, other):
+            return self.__view <= other
+
+        def __lt__(self, other):
+            return self.__view < other
+
+        def __ne__(self, other):
+            return self.__view != other
+
+        def __hash__(self):
+            # raises
+            return hash(self.__view)
+
+        def __delitem__(self, key):
+            # raises
+            del self.__view[key]
+
+        def __len__(self):
+            return len(self.__view)
+
+        # To access format, itemsize, ndim, readonly, shape, strides,
+        # suboffsets, tobytes, tolist.
+        def __getattr__(self, name):
+            return getattr(self.__view, name)
+
+        # Actual payload of this class.
+        def __getitem__(self, key):
+            value = self.__view[key]
+            if isinstance(value, memoryview):
+                return self.__class__(value)
+            return ord(value)
+
+        def __setitem__(self, key, value):
+            if isinstance(value, (int, long)):
+                value = chr(value)
+            else:
+                value = ''.join(chr(x) for x in value)
+            self.__view[key] = value
+
 # pylint: disable=undefined-variable
 CONTROL_SETUP = BYTE * CONTROL_SETUP_SIZE
 # pylint: enable=undefined-variable
@@ -401,7 +462,9 @@ class USBTransfer(object):
         self.__initialized = False
         self.__transfer_buffer = string_buffer
         # pylint: disable=undefined-variable
-        self.__transfer_py_buffer = transfer_py_buffer[CONTROL_SETUP_SIZE:]
+        self.__transfer_py_buffer = integer_memoryview(
+            transfer_py_buffer,
+        )[CONTROL_SETUP_SIZE:]
         # pylint: enable=undefined-variable
         self.__user_data = user_data
         libusb1.libusb_fill_control_setup(
@@ -1176,8 +1239,8 @@ class USBDeviceHandle(object):
 
     def getDevice(self):
         """
-        Get an USBDevice instance for the device accessed through this handle.
-        Useful for example to query its configurations.
+        Get an USBDevice instance for the device accessed through this handle,
+        to access to the descriptors available in OS cache.
         """
         return self.__device
 
@@ -1199,6 +1262,30 @@ class USBDeviceHandle(object):
             libusb1.libusb_set_configuration(self.__handle, configuration),
         )
 
+    def getManufacturer(self):
+        """
+        Get device's manufaturer name.
+        """
+        return self.getASCIIStringDescriptor(
+            self.__device.device_descriptor.iManufacturer,
+        )
+
+    def getProduct(self):
+        """
+        Get device's product name.
+        """
+        return self.getASCIIStringDescriptor(
+            self.__device.device_descriptor.iProduct,
+        )
+
+    def getSerialNumber(self):
+        """
+        Get device's serial number.
+        """
+        return self.getASCIIStringDescriptor(
+            self.__device.device_descriptor.iSerialNumber,
+        )
+
     def claimInterface(self, interface):
         """
         Claim (= get exclusive access to) given interface number. Required to
@@ -1317,6 +1404,8 @@ class USBDeviceHandle(object):
         Return value is a unicode string.
         Return None if there is no such descriptor on device.
         """
+        if descriptor == 0:
+            return None
         descriptor_string = bytearray(STRING_LENGTH)
         try:
             received = mayRaiseUSBError(libusb1.libusb_get_string_descriptor(
@@ -1342,6 +1431,8 @@ class USBDeviceHandle(object):
         Return value is a unicode string.
         Return None if there is no such descriptor on device.
         """
+        if descriptor == 0:
+            return None
         descriptor_string = bytearray(STRING_LENGTH)
         try:
             received = mayRaiseUSBError(libusb1.libusb_get_string_descriptor_ascii(
@@ -1413,9 +1504,13 @@ class USBDeviceHandle(object):
 
     def _bulkTransfer(self, endpoint, data, length, timeout):
         transferred = c_int()
-        mayRaiseUSBError(libusb1.libusb_bulk_transfer(
-            self.__handle, endpoint, data, length, byref(transferred), timeout,
-        ))
+        try:
+            mayRaiseUSBError(libusb1.libusb_bulk_transfer(
+                self.__handle, endpoint, data, length, byref(transferred), timeout,
+            ))
+        except USBErrorTimeout as exception:
+            exception.transferred = transferred.value
+            raise
         return transferred.value
 
     def bulkWrite(self, endpoint, data, timeout=0):
@@ -1430,6 +1525,10 @@ class USBDeviceHandle(object):
         interface (ex: bytearray) for the "data" parameter.
 
         Returns the number of bytes actually sent.
+
+        May raise an exception from the USBError family. USBErrorTimeout
+        exception has a "transferred" property giving the number of bytes sent
+        up to the timeout.
         """
         # pylint: disable=undefined-variable
         endpoint = (endpoint & ~ENDPOINT_DIR_MASK) | ENDPOINT_OUT
@@ -1448,19 +1547,36 @@ class USBDeviceHandle(object):
         interface (ex: bytearray) for the "data" parameter.
 
         Returns received data.
+
+        May raise an exception from the USBError family. USBErrorTimeout
+        exception has a "received" property giving the bytes received up to the
+        timeout.
         """
         # pylint: disable=undefined-variable
         endpoint = (endpoint & ~ENDPOINT_DIR_MASK) | ENDPOINT_IN
         # pylint: enable=undefined-variable
         data, data_buffer = create_binary_buffer(length)
-        transferred = self._bulkTransfer(endpoint, data, length, timeout)
+        try:
+            transferred = self._bulkTransfer(endpoint, data, length, timeout)
+        except USBErrorTimeout as exception:
+            exception.received = data_buffer[:exception.transferred]
+            raise
         return data_buffer[:transferred]
 
     def _interruptTransfer(self, endpoint, data, length, timeout):
         transferred = c_int()
-        mayRaiseUSBError(libusb1.libusb_interrupt_transfer(
-            self.__handle, endpoint, data, length, byref(transferred), timeout,
-        ))
+        try:
+            mayRaiseUSBError(libusb1.libusb_interrupt_transfer(
+                self.__handle,
+                endpoint,
+                data,
+                length,
+                byref(transferred),
+                timeout,
+            ))
+        except USBErrorTimeout as exception:
+            exception.transferred = transferred.value
+            raise
         return transferred.value
 
     def interruptWrite(self, endpoint, data, timeout=0):
@@ -1475,6 +1591,10 @@ class USBDeviceHandle(object):
         interface (ex: bytearray) for the "data" parameter.
 
         Returns the number of bytes actually sent.
+
+        May raise an exception from the USBError family. USBErrorTimeout
+        exception has a "transferred" property giving the number of bytes sent
+        up to the timeout.
         """
         # pylint: disable=undefined-variable
         endpoint = (endpoint & ~ENDPOINT_DIR_MASK) | ENDPOINT_OUT
@@ -1487,18 +1607,31 @@ class USBDeviceHandle(object):
         Synchronous interrupt write.
         timeout: in milliseconds, how long to wait for data. Set to 0 to
           disable.
-        See interruptRead for other parameters description.
+        See interruptWrite for other parameters description.
 
         To avoid memory copies, use an object implementing the writeable buffer
         interface (ex: bytearray) for the "data" parameter.
 
         Returns received data.
+
+        May raise an exception from the USBError family. USBErrorTimeout
+        exception has a "received" property giving the bytes received up to the
+        timeout.
         """
         # pylint: disable=undefined-variable
         endpoint = (endpoint & ~ENDPOINT_DIR_MASK) | ENDPOINT_IN
         # pylint: enable=undefined-variable
         data, data_buffer = create_binary_buffer(length)
-        transferred = self._interruptTransfer(endpoint, data, length, timeout)
+        try:
+            transferred = self._interruptTransfer(
+                endpoint,
+                data,
+                length,
+                timeout,
+            )
+        except USBErrorTimeout as exception:
+            exception.received = data_buffer[:exception.transferred]
+            raise
         return data_buffer[:transferred]
 
     def getTransfer(self, iso_packets=0):
@@ -1720,6 +1853,10 @@ class USBEndpoint(object):
 class USBDevice(object):
     """
     Represents a USB device.
+
+    Exposes USB descriptors which are available from OS without needing to get
+    a USBDeviceHandle: device descriptor, configuration descriptors, interface
+    descriptors, setting descritptors, endpoint descriptors.
     """
 
     __configuration_descriptor_list = ()
@@ -1942,39 +2079,57 @@ class USBDevice(object):
     def getSupportedLanguageList(self):
         """
         Get the list of language ids device has string descriptors for.
+        Note: opens the device temporarily and uses synchronous API.
         """
         return self.open().getSupportedLanguageList()
 
-    def _getStringDescriptor(self, descriptor, lang_id):
-        if descriptor:
-            return self.open().getStringDescriptor(descriptor, lang_id)
-
-    def _getASCIIStringDescriptor(self, descriptor):
-        if descriptor:
-            return self.open().getASCIIStringDescriptor(descriptor)
-
     def getManufacturer(self):
         """
         Get device's manufaturer name.
-        Note: opens the device temporarily.
+
+        Shortcut for .open().getManufacturer() .
         """
-        return self._getASCIIStringDescriptor(
-            self.device_descriptor.iManufacturer)
+        return self.open().getManufacturer()
+
+    def getManufacturerDescriptor(self):
+        """
+        Get the string index of device's manufacturer.
+        You can pass this value to USBHandle.getASCIIStringDescriptor to get
+        the actual manufacturer string.
+        """
+        return self.device_descriptor.iManufacturer
 
     def getProduct(self):
         """
         Get device's product name.
-        Note: opens the device temporarily.
+
+        Shortcut for .open().getProduct() .
         """
-        return self._getASCIIStringDescriptor(self.device_descriptor.iProduct)
+        return self.open().getProduct()
+
+    def getProductDescriptor(self):
+        """
+        Get the string index of device's product name.
+        You can pass this value to USBHandle.getASCIIStringDescriptor to get
+        the actual product name string.
+        """
+        return self.device_descriptor.iProduct
 
     def getSerialNumber(self):
         """
         Get device's serial number.
-        Note: opens the device temporarily.
+
+        Shortcut for .open().getSerialNumber() .
         """
-        return self._getASCIIStringDescriptor(
-            self.device_descriptor.iSerialNumber)
+        return self.open().getSerialNumber()
+
+    def getSerialNumberDescriptor(self):
+        """
+        Get the string index of device's serial number.
+        You can pass this value to USBHandle.getASCIIStringDescriptor to get
+        the actual serial number string.
+        """
+        return self.device_descriptor.iSerialNumber
 
     def getNumConfigurations(self):
         """
@@ -2022,8 +2177,9 @@ class USBContext(object):
     __removed_cb = None
     __poll_cb_user_data = None
     __libusb_set_pollfd_notifiers = libusb1.libusb_set_pollfd_notifiers
-    __null_pointer = POINTER(None)
+    __null_pointer = c_void_p()
     __KeyError = KeyError
+    __cast = staticmethod(cast)
     __auto_open = True
 
     # pylint: disable=no-self-argument,protected-access
@@ -2321,7 +2477,11 @@ class USBContext(object):
         self.__removed_cb = removed_cb
         self.__poll_cb_user_data = user_data
         self.__libusb_set_pollfd_notifiers(
-            self.__context_p, added_cb, removed_cb, user_data)
+            self.__context_p,
+            self.__cast(added_cb, libusb1.libusb_pollfd_added_cb_p),
+            self.__cast(removed_cb, libusb1.libusb_pollfd_removed_cb_p),
+            user_data,
+        )
 
     @_validContext
     def getNextTimeout(self):
@@ -2451,6 +2611,9 @@ class USBContext(object):
             HOTPLUG_EVENT_DEVICE_LEFT
         Callback must return whether it must be unregistered (any true value
         to be unregistered, any false value to be kept registered).
+
+        Note: given callback will be invoked during event handling, meaning
+        it cannot call any synchronous libusb function.
         """
         def wrapped_callback(context_p, device_p, event, _):
             assert addressof(context_p.contents) == addressof(
